#define F_CPU 4800000UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "HAL.h"
#include "main.h"
#include "soft_timer.h"

volatile unsigned char
	max_inputs = 4,			//    (2, 3, 4)
	i,
	clicks		= 0;		//   

unsigned char
	task_flags = 0,
	active_input = IN1,
	temp = 0,
	key_state = OPEN;

//   
ISR(TIM0_COMPA_vect)
{
	for (i = 0; i != MAX_NUMBERS_OF_TIMERS; i++)		//   
	{
		if (soft_timer[i].number == 255)
			continue;									//     -  
		
		if(soft_timer[i].time != 0)						//    ,    .
		{
			soft_timer[i].time--;						//       .
		}
		else
		{
			task_flags |= (1 << soft_timer[i].number);	//   ?     
			soft_timer[i].number = 255;					//      -  .
		}
	}
}

//   
ISR(PCINT0_vect)
{
	//    
	set_task(SCAN_KEY, DEBOUNCE_TIME);			
}

//  
inline void init_system(void)
{
	//------------------------------------------------------------------------
	//  
	//------------------------------------------------------------------------
	
	REL_PORT &= ~((1 << REL_IN1) | (1 << REL_IN2) | (1 << REL_IN3) | (1 << REL_IN4));
	
	// 
	KEY_DDR &= ~(1 << KEY);
	
	//------------------------------------------------------------------------
	//  
	//------------------------------------------------------------------------
	
	//   T0    (T = 1 ms)
	TCCR0A |= (1 << WGM01);					//  CTC
	TCCR0B |= (1 << CS01) | (1 << CS00);	//  1/64
	OCR0A = 75;								// 4 800 000 / (64 * 75) = 1000 Hz (T = 1 ms)
	TIMSK0 |= (1 << OCIE0A);				//     T0A
		
	//------------------------------------------------------------------------
	//   Pin Change
	//------------------------------------------------------------------------
	
	//   Pin Change Interrupt
	GIMSK  |= (1 << PCIE);
	
	//   
	PCMSK |= (1 << PCINT4);				//  SELECTOR
}

//     
void task_scan_key(void)
{
	reset_flag(SCAN_KEY);
	
	//    SELECT
	if (KEY_PIN & (1 << KEY))
	{
		clicks++;
	}
}

//   
void task_input_enable(void)
{
	reset_flag(INP_ENABLE);
	
	switch (active_input) {
		case IN1	: REL_PORT |= (1 << REL_IN1); break;
		case IN2	: REL_PORT |= (1 << REL_IN2); break;
		case IN3	: REL_PORT |= (1 << REL_IN3); break;
		case IN4	: REL_PORT |= (1 << REL_IN4); break;
		default     : REL_PORT |= (1 << REL_IN1); break;
	}
}

// --------------------------------    EEPROM
unsigned char read_EEPROM(unsigned int addr)
{
	while (EECR & (1 << EEPE));
	EEARL = addr;
	EECR |= (1 << EERE);
	return EEDR;
}

// --------------------------------    EEPROM
void update_EEPROM(unsigned int addr, unsigned char data)
{
	unsigned char temp_data;
	temp_data = read_EEPROM(SAVE_ADDR);
	if (temp_data != data)
	{
		while (EECR & (1 << EEPE));
		EECR = (0 << EEPM1) | (0 >> EEPM0);
		EEARL = addr;
		EEDR = data;
		EECR |= (1 << EEMPE);
		EECR |= (1 << EEPE);
	}
}

void button_processing(void)
{
	if (clicks > 0)
	{
		temp = active_input;
		switch (temp) {
			case IN1	: active_input = active_input << 1; break;
			case IN2	: if (max_inputs == 2) active_input = IN1; else active_input = active_input << 1; break;
			case IN3	: if (max_inputs == 3) active_input = IN1; else active_input = active_input << 1; break;
			case IN4	: active_input = IN1;				break;
			default     : active_input = IN1;				break;
		}
		clicks--;

		if (clicks == 0)	//   ,   
		{
			
			inputs_disable;
			set_task(INP_ENABLE, DELAY);
			update_EEPROM(SAVE_ADDR, active_input);
		}
	}
}

//    
void pr_scan_task_flags(void)
{
	if (task_flags & (1 << SCAN_KEY))
	{
		task_scan_key();
	}
	
	if (task_flags & (1 << INP_ENABLE))
	{
		task_input_enable();
	}
}

int main(void)
{
	//     
	REL_DDR &= ~((1 << REL_IN1) | (1 << REL_IN2) | (1 << REL_IN3) | (1 << REL_IN4));
	REL_PORT |= ((1 << REL_IN1) | (1 << REL_IN2) | (1 << REL_IN3) | (1 << REL_IN4));
	
	if (REL_PIN & (1 << REL_IN3))
	{
		max_inputs = 2;
		REL_DDR |= (1 << REL_IN1) | (1 << REL_IN2);
	} 
	else if (REL_PIN & (1 << REL_IN4))
	{
		max_inputs = 3;
		REL_DDR |= (1 << REL_IN1) | (1 << REL_IN2) | (1 << REL_IN3);
	}
	else
	{
		REL_DDR |= (1 << REL_IN1) | (1 << REL_IN2) | (1 << REL_IN3) | (1 << REL_IN4);
	}
	
	//   
    init_system();
	inputs_disable;
	_delay_ms(INIT_DELAY);
	
	unsigned char EEPROM_input;

	EEPROM_input = read_EEPROM(SAVE_ADDR);					//      EEPROM
	switch (EEPROM_input) {									//   
		case IN1	: active_input = EEPROM_input; break;
		case IN2	: active_input = EEPROM_input; break;
		case IN3	: if (max_inputs <= 2) active_input = IN1; else active_input = EEPROM_input; break;
		case IN4	: if (max_inputs <= 3) active_input = IN1; else active_input = EEPROM_input; break;
		default     : active_input = IN1;		   break;	//    ,   
	}
	
	task_input_enable();
	sei(); //   
	
	while (1) 
    {		
		button_processing();
		pr_scan_task_flags();											//   
    }
}